package observer;

import java.util.Observable;
import java.util.Iterator;

public aspect ObservingOpen extends ObserverProtocol {
	declare parents: Flower implements Subject;
	declare parents: (Bee || Hummingbird) implements Observer;

	protected pointcut subjectChange(Subject subject):
		execution(void open()) && this(subject);
    after(Subject s): subjectChange(s) {
        Iterator iter = getObservers(s).iterator();
        while ( iter.hasNext() ) {
            updateObserver(s, ((Observer)iter.next()));
        }
    }
	protected void updateObserver(Subject s, Observer o) { }

	public abstract boolean Subject.isOpen();
	public abstract void Observer.breakfastTime();

	static class OpenNotifier extends Observable {
		private Subject _enclosing;
		private boolean alreadyOpen = false;
		public OpenNotifier(Subject flower) {
			_enclosing = flower;
		}
		public void notifyObservers() {
			if(_enclosing.isOpen() && !this.alreadyOpen) {
				this.setChanged();
				super.notifyObservers();
				this.alreadyOpen = true;
			}
		}
		public void close() {
			this.alreadyOpen = false;
		}
	}
	static class OpenObserver implements java.util.Observer {
		private Observer _enclosing;
		public OpenObserver(Observer enclosing) {
			_enclosing = enclosing;
		}
		public void update(Observable ob, Object a) {
			_enclosing.breakfastTime();
		}
	}
	private	OpenNotifier Subject.oNotify = new OpenNotifier(this);
	private OpenObserver Observer.openObsrv = new OpenObserver(this);

	public Observable Subject.opening() {
		return oNotify;
	}
//	pointcut flowerOpen(Subject flower):
//		execution(void open()) && this(flower);
	after(Subject flower) returning : subjectChange(flower) {
		flower.oNotify.notifyObservers();
	}
	pointcut flowerClose(Subject flower):
		execution(void close()) && this(flower);
	after(Subject flower): flowerClose(flower) {
		flower.oNotify.close();
	}
	public java.util.Observer Observer.openObserver() { 
		return openObsrv; 
	}
}
