package observer;

import java.util.Observable;
import java.util.Iterator;

public aspect ObservingOpen extends ObserverProtocol {
	declare parents: Flower implements Subject;
	declare parents: (Bee || Hummingbird) implements Observer;

	protected pointcut subjectChange(Subject subject):
		execution(void open()) && this(subject);
	after(Subject s): subjectChange(s) {
		Iterator iter = getObservers(s).iterator();
		while ( iter.hasNext() ) {
			updateObserver(s, ((Observer)iter.next()));
		}
//		Flower f = (Flower)s;
//		if(f.isOpen() && !f.alreadyOpen) {
//			Iterator iter = getObservers(s).iterator();
//			while ( iter.hasNext() ) {
//				updateObserver(s, ((Observer)iter.next()));
//			}
//		}
	}
	after(Subject flower) returning : subjectChange(flower) {
		flower.oNotify.notifyObservers();
	}
	pointcut flowerClose(Subject flower):
		execution(void close()) && this(flower);
	after(Subject flower): flowerClose(flower) {
		flower.oNotify.close();
	}
	after(Subject subject): flowerClose(subject) {
		if (subject instanceof Flower) {
			((Flower)subject).alreadyOpen = false;
		}
	}

	private boolean Subject.alreadyOpen = false;

	protected void updateObserver(Subject s, Observer o) {
		if (s instanceof Flower) {
			Flower flower = (Flower)s;
			if(flower.isOpen() && !flower.alreadyOpen) {
				o.breakfastTime();
				flower.alreadyOpen = true;
			}
		}
	}
	public abstract boolean Subject.isOpen();
	public abstract void Observer.breakfastTime();

	static class OpenNotifier extends Observable {
		private Subject _enclosing;
		private boolean alreadyOpen = false;
		public OpenNotifier(Subject flower) {
			_enclosing = flower;
		}
		public void notifyObservers() {
			if(_enclosing.isOpen() && !this.alreadyOpen) {
				this.setChanged();
				super.notifyObservers();
				this.alreadyOpen = true;
			}
		}
		public void close() {
			this.alreadyOpen = false;
		}
	}
	static class OpenObserver implements java.util.Observer {
		private Observer _enclosing;
		public OpenObserver(Observer enclosing) {
			_enclosing = enclosing;
		}
		public void update(Observable ob, Object a) {
			_enclosing.breakfastTime();
		}
	}
	private	OpenNotifier Subject.oNotify = new OpenNotifier(this);
	private OpenObserver Observer.openObsrv = new OpenObserver(this);

	public Observable Subject.opening() {
		return oNotify;
	}
	public java.util.Observer Observer.openObserver() { 
		return openObsrv; 
	}
}
