package observer;

import junit.framework.*;
import util.*;

public class TestObservedFlower extends TestCase {
	Flower f = new Flower();
	Bee ba = new Bee("A"), 
		bb = new Bee("B");
	Hummingbird
		hx = new Hummingbird("X"), 
		hy = new Hummingbird("Y");
	public void setUp() {
		f = new Flower();
		ba = new Bee("A"); 
		bb = new Bee("B");
		hx = new Hummingbird("X"); 
		hy = new Hummingbird("Y");
	}
	public void tearDown() {
		f = null;
		ba = null; 
		bb = null;
		hx = null; 
		hy = null;
		SpecificOutputCapture.clearResult();
	}
	private static String expectedOutput() {
		return 
		"Flower open.\n" +
		"Hummingbird X's breakfast time!\n" +
		"Bee B's breakfast time!\n" +
		"Bee A's breakfast time!\n" +
		"Flower open.\n" +
		"Flower close.\n" +
		"Hummingbird Y's bed time!\n" +
		"Hummingbird X's bed time!\n" +
		"Bee B's bed time!\n" +
		"Flower close.\n" +
		"Flower open.\n" +
		"Flower close.\n" +
		"Hummingbird Y's bed time!\n" +
		"Hummingbird X's bed time!\n" +
		"Bee B's bed time!\n";
	}
	public void test() {
		ObservingOpen.aspectOf().addObserver(f, hy);
		ObservingOpen.aspectOf().addObserver(f, hx);
		ObservingOpen.aspectOf().addObserver(f, bb);
		ObservingOpen.aspectOf().addObserver(f, ba);

		ObservingClose.aspectOf().addObserver(f, hy);
		ObservingClose.aspectOf().addObserver(f, hx);
		ObservingClose.aspectOf().addObserver(f, bb);
		ObservingClose.aspectOf().addObserver(f, ba);

		// Hummingbird Y decides to sleep in:
		ObservingOpen.aspectOf().removeObserver(f, hy);

		// A change that interests observers:
		f.open();
		f.open(); // It's already open, no change.

		// Bee A doesn't want to go to bed:
		ObservingClose.aspectOf().removeObserver(f, ba);

		f.close();
		f.close(); // It's already closed; no change
		ObservingOpen.aspectOf().clearObservers(f);

		f.open();
		f.close();
		Assert.assertEquals(expectedOutput(), SpecificOutputCapture.getResult());
	}
	/**This inner aspect supresses output from objects within this package
	 * when they do NOT originate from within the control flow of main()    */
	private static aspect SpecificOutputSuppress extends OutputSupress {
		protected pointcut scopeException():
			cflow(execution(
				public static void TestObservedFlower*.main(String[])
				));
	}
	private static aspect SpecificOutputCapture extends OutputCapture {
		declare precedence:
			TestObservedFlower.SpecificOutputCapture,
			TestObservedFlower.SpecificOutputSuppress;
		public pointcut printScope():
			!within(TestObservedFlower.SpecificOutputCapture);
	}
	public static void main(String args[]) {
		junit.textui.TestRunner.run(TestObservedFlower.class);
	}
}
