package util;

/**
 * This aspect supresses output from objects when they do NOT originate
 * from joinpoints captured by the abstract pointcut scopeException().
 * 
 * @author Miguel Pessoa Monteiro
 */
public abstract aspect OutputSupress {
	/**
	 * Specifies a set of joinpoits that are exempt from the
	 * output supression. 
	 */
	protected abstract pointcut scopeException();

	/**
	 * Captures all calls to System.out.print*(..)
	 */
	public pointcut allSysOutPrintsWithException():
		Pointcuts.allCalls2SystemOutPrints()
		&& !scopeException();

	/**
	 * Deactivate messages to the console from System.out.print*()
	 * methods
	 */
	void around(): allSysOutPrintsWithException() {
		//null op
	}
}
